use crate::db::{db_new_user, db_user_login};
use crate::jwt::Authentication;
// use crate::jwt::token_factory;
use crate::state::CaptchaState;
use crate::types::{AuthResponse, LoginData, RegisterData};
use crate::ServerState;

use actix_web::web::Data;
use actix_web::{post, web::Json, HttpResponse, Responder, Result};
use biosvg::BiosvgBuilder;
use log::*;

#[post("/register")]
pub async fn register(
    data: Json<RegisterData>,
    state: Data<ServerState>,
    captcha_state: Data<CaptchaState>,
    auth: Data<Authentication>,
) -> Result<impl Responder> {
    if !captcha_state
        .capthca_db
        .lock()
        .unwrap()
        .remove(&data.captcha)
    {
        info!("User failed to register, captcha was wrong");
        return Ok(HttpResponse::BadRequest().json("Error"));
    }

    match db_new_user(data.username.clone(), data.password.clone(), &state.pool).await {
        Some(user) => {
            info!("User: {} registered", &user.username);
            Ok(HttpResponse::Ok().json(AuthResponse {
                username: user.username.clone(),
                token: auth.encode(&user.username).unwrap(),
            }))
        }
        None => {
            info!("User \"{}\" already exists", data.username);
            return Ok(HttpResponse::BadRequest().json("Error"));
        }
    }
}

#[post("/login")]
pub async fn login(
    data: Json<LoginData>,
    state: Data<ServerState>,
    auth: Data<Authentication>,
) -> Result<impl Responder> {
    let result = db_user_login(data.username.clone(), data.password.clone(), &state.pool).await;

    match result {
        Some(_) => {
            return Ok(HttpResponse::Ok().json(AuthResponse {
                username: data.username.clone(),
                token: auth.encode(&data.username).unwrap(),
            }));
        }
        None => {
            info!("User \"{}\" failed to log in", data.username);
            return Ok(HttpResponse::BadRequest().json("Error"));
        }
    }
}

/// Request a captcha from the captcha service
#[allow(unreachable_code, unused_variables)]
#[post("/captcha")]
pub async fn captcha_request(cstate: Data<CaptchaState>) -> Result<impl Responder> {
    unimplemented!("Captcha is currently disabled");
    return Ok(HttpResponse::InternalServerError().json("Error"));

    // This might block the thread a bit too long
    let (answer, svg) = get_captcha();

    // let id = rand_core::OsRng.next_u32() as i32;

    // let cresponse = CaptchaResponse {
    //     captcha_svg: svg.clone(),
    //     captcha_id: id,
    // };

    // This is bad in about every way i can think of
    // It might just be better to hit the database every time, and let the database
    // handle rng and maybe set a trigger to delete old captchas
    // match cstate.capthca_db.lock() {
    //     Ok(mut db) => {
    //         if (db.len() as i32) > 100 {
    //             // To prevent the database from growing too large
    //             // Replace with a proper LRU cache or circular buffer
    //             db.remove(&(id % 100)); // This is terrible
    //         }
    //         db.insert(id, answer.clone()); // We do not care about collisions
    //         return Ok(HttpResponse::Ok().json(cresponse));
    //     }
    //     Err(_) => {
    //         // This shouldnt happen
    //         error!("Failed to lock captcha database");
    //         return Ok(HttpResponse::InternalServerError().json("Error"));
    //     }
    // }
}

/// Returns a new captcha in the form of a tuple (answer, svg)
fn get_captcha() -> (String, String) {
    BiosvgBuilder::new()
        .length(4)
        .difficulty(6)
        .colors(vec![
            // Feel free to change these
            "#0078D6".to_string(),
            "#aa3333".to_string(),
            "#f08012".to_string(),
            "#33aa00".to_string(),
            "#aa33aa".to_string(),
        ])
        .build()
        .unwrap()
}
